@@ -5,10 +5,15 @@
 # the BSD License: http://www.opensource.org/licenses/bsd-license.php
 
 import os
+import os.path
 import sys
 import select
 import logging
 import threading
+import errno
+import mmap
+
+from contextlib import contextmanager
 from subprocess import (
     call,
     Popen,
@@ -26,6 +31,7 @@ from git.compat import (
     string_types,
     defenc,
     PY3,
+    bchr,
     # just to satisfy flake8 on py3
     unicode
 )
@@ -41,6 +47,13 @@ __all__ = ('Git', )
 if sys.platform != 'win32':
     WindowsError = OSError
 
+if PY3:
+    _bchr = bchr
+else:
+    def _bchr(c):
+        return c
+# get custom byte character handling
+
 
 # ==============================================================================
 ## @name Utilities
@@ -58,57 +71,77 @@ def handle_process_output(process, stdout_handler, stderr_handler, finalizer):
     :param stdout_handler: f(stdout_line_string), or None
     :param stderr_hanlder: f(stderr_line_string), or None
     :param finalizer: f(proc) - wait for proc to finish"""
-    def read_line_fast(stream):
-        return stream.readline()
-
-    def read_line_slow(stream):
+    def parse_lines_from_buffer(fno, buf):
         line = b''
-        while True:
-            char = stream.read(1)       # reads individual single byte strings
-            if not char:
-                break
+        bi = 0
+        lb = len(buf)
+        while bi < lb:
+            char = _bchr(buf[bi])
+            bi += 1
 
             if char in (b'\r', b'\n') and line:
-                break
+                yield bi, line
+                line = b''
             else:
                 line += char
             # END process parsed line
         # END while file is not done reading
-        return line
     # end
 
-    def dispatch_line(fno):
-            stream, handler, readline = fdmap[fno]
-            # this can possibly block for a while, but since we wake-up with at least one or more lines to handle,
-            # we are good ...
-            line = readline(stream).decode(defenc)
-            if line and handler:
-                try:
-                    handler(line)
-                except Exception:
-                    # Keep reading, have to pump the lines empty nontheless
-                    log.error("Line handler exception on line: %s", line, exc_info=True)
-                # end
-            return line
+    def read_lines_from_fno(fno, last_buf_list):
+        buf = os.read(fno, mmap.PAGESIZE)
+        buf = last_buf_list[0] + buf
+
+        bi = 0
+        for bi, line in parse_lines_from_buffer(fno, buf):
+            yield line
+        # for each line to parse from the buffer
+
+        # keep remainder
+        last_buf_list[0] = buf[bi:]
+
+    def dispatch_single_line(line, handler):
+        line = line.decode(defenc)
+        if line and handler:
+            try:
+                handler(line)
+            except Exception:
+                # Keep reading, have to pump the lines empty nontheless
+                log.error("Line handler exception on line: %s", line, exc_info=True)
+            # end
         # end dispatch helper
+    # end single line helper
+
+    def dispatch_lines(fno, handler, buf_list):
+        lc = 0
+        for line in read_lines_from_fno(fno, buf_list):
+            dispatch_single_line(line, handler)
+            lc += 1
+        # for each line
+        return lc
     # end
 
-    def deplete_buffer(fno, wg=None):
+    def deplete_buffer(fno, handler, buf_list, wg=None):
         while True:
-            line = dispatch_line(fno)
-            if not line:
+            line_count = dispatch_lines(fno, handler, buf_list)
+            if line_count == 0:
                 break
         # end deplete buffer
+
+        if buf_list[0]:
+            dispatch_single_line(buf_list[0], handler)
+        # end
+
         if wg:
             wg.done()
     # end
 
-    fdmap = {process.stdout.fileno(): (process.stdout, stdout_handler, read_line_fast),
-             process.stderr.fileno(): (process.stderr, stderr_handler, read_line_slow)}
+    fdmap = {process.stdout.fileno(): (stdout_handler, [b'']),
+             process.stderr.fileno(): (stderr_handler, [b''])}
 
     if hasattr(select, 'poll'):
         # poll is preferred, as select is limited to file handles up to 1024 ... . This could otherwise be
-        # an issue for us, as it matters how many handles or own process has
+        # an issue for us, as it matters how many handles our own process has
         poll = select.poll()
         READ_ONLY = select.POLLIN | select.POLLPRI | select.POLLHUP | select.POLLERR
         CLOSED = select.POLLHUP | select.POLLERR
@@ -119,12 +152,20 @@ def handle_process_output(process, stdout_handler, stderr_handler, finalizer):
         closed_streams = set()
         while True:
             # no timeout
-            poll_result = poll.poll()
+
+            try:
+                poll_result = poll.poll()
+            except select.error as e:
+                if e.args[0] == errno.EINTR:
+                    continue
+                raise
+            # end handle poll exception
+
             for fd, result in poll_result:
                 if result & CLOSED:
                     closed_streams.add(fd)
                 else:
-                    dispatch_line(fd)
+                    dispatch_lines(fd, *fdmap[fd])
                 # end handle closed stream
             # end for each poll-result tuple
 
@@ -134,18 +175,22 @@ def handle_process_output(process, stdout_handler, stderr_handler, finalizer):
         # end endless loop
 
         # Depelete all remaining buffers
-        for fno in fdmap.keys():
-            deplete_buffer(fno)
+        for fno, (handler, buf_list) in fdmap.items():
+            deplete_buffer(fno, handler, buf_list)
         # end for each file handle
+
+        for fno in fdmap.keys():
+            poll.unregister(fno)
+        # end don't forget to unregister !
     else:
         # Oh ... probably we are on windows. select.select() can only handle sockets, we have files
         # The only reliable way to do this now is to use threads and wait for both to finish
         # Since the finalizer is expected to wait, we don't have to introduce our own wait primitive
         # NO: It's not enough unfortunately, and we will have to sync the threads
         wg = WaitGroup()
-        for fno in fdmap.keys():
+        for fno, (handler, buf_list) in fdmap.items():
             wg.add(1)
-            t = threading.Thread(target=lambda: deplete_buffer(fno, wg))
+            t = threading.Thread(target=lambda: deplete_buffer(fno, handler, buf_list, wg))
             t.start()
         # end
         # NOTE: Just joining threads can possibly fail as there is a gap between .start() and when it's
@@ -180,7 +225,7 @@ class Git(LazyMixin):
         Set its value to 'full' to see details about the returned values.
     """
     __slots__ = ("_working_dir", "cat_file_all", "cat_file_header", "_version_info",
-                 "_git_options")
+                 "_git_options", "_environment")
 
     # CONFIGURATION
     # The size in bytes read from stdout when copying git's output to another stream
@@ -252,7 +297,7 @@ class Git(LazyMixin):
             :raise GitCommandError: if the return status is not 0"""
             status = self.proc.wait()
             if status != 0:
-                raise GitCommandError(self.args, status, self.proc.stderr.read().decode(defenc))
+                raise GitCommandError(self.args, status, self.proc.stderr.read())
             # END status handling
             return status
     # END auto interrupt
@@ -370,6 +415,9 @@ class Git(LazyMixin):
         self._working_dir = working_dir
         self._git_options = ()
 
+        # Extra environment variables to pass to git commands
+        self._environment = {}
+
         # cached command slots
         self.cat_file_header = None
         self.cat_file_all = None
@@ -493,6 +541,8 @@ class Git(LazyMixin):
         # Start the process
         env = os.environ.copy()
         env["LC_MESSAGES"] = "C"
+        env.update(self._environment)
+
         proc = Popen(command,
                      env=env,
                      cwd=cwd,
@@ -536,11 +586,16 @@ class Git(LazyMixin):
 
         if self.GIT_PYTHON_TRACE == 'full':
             cmdstr = " ".join(command)
+
+            def as_text(stdout_value):
+                return not output_stream and stdout_value.decode(defenc) or '<OUTPUT_STREAM>'
+            # end
+
             if stderr_value:
                 log.info("%s -> %d; stdout: '%s'; stderr: '%s'",
-                         cmdstr, status, stdout_value.decode(defenc), stderr_value.decode(defenc))
+                         cmdstr, status, as_text(stdout_value), stderr_value.decode(defenc))
             elif stdout_value:
-                log.info("%s -> %d; stdout: '%s'", cmdstr, status, stdout_value.decode(defenc))
+                log.info("%s -> %d; stdout: '%s'", cmdstr, status, as_text(stdout_value))
             else:
                 log.info("%s -> %d", cmdstr, status)
         # END handle debug printing
@@ -560,6 +615,57 @@ class Git(LazyMixin):
         else:
             return stdout_value
 
+    def environment(self):
+        return self._environment
+
+    def update_environment(self, **kwargs):
+        """
+        Set environment variables for future git invocations. Return all changed
+        values in a format that can be passed back into this function to revert
+        the changes:
+
+        ``Examples``::
+
+            old_env = self.update_environment(PWD='/tmp')
+            self.update_environment(**old_env)
+
+        :param kwargs: environment variables to use for git processes
+        :return: dict that maps environment variables to their old values
+        """
+        old_env = {}
+        for key, value in kwargs.items():
+            # set value if it is None
+            if value is not None:
+                if key in self._environment:
+                    old_env[key] = self._environment[key]
+                else:
+                    old_env[key] = None
+                self._environment[key] = value
+            # remove key from environment if its value is None
+            elif key in self._environment:
+                old_env[key] = self._environment[key]
+                del self._environment[key]
+        return old_env
+
+    @contextmanager
+    def custom_environment(self, **kwargs):
+        """
+        A context manager around the above ``update_environment`` method to restore the
+        environment back to its previous state after operation.
+
+        ``Examples``::
+
+            with self.custom_environment(GIT_SSH='/bin/ssh_wrapper'):
+                repo.remotes.origin.fetch()
+
+        :param kwargs: see update_environment
+        """
+        old_env = self.update_environment(**kwargs)
+        try:
+            yield
+        finally:
+            self.update_environment(**old_env)
+
     def transform_kwargs(self, split_single_char_options=False, **kwargs):
         """Transforms Python style kwargs into git command line options."""
         args = list()
@@ -683,7 +789,7 @@ class Git(LazyMixin):
                         import warnings
                         msg = "WARNING: Automatically switched to use git.cmd as git executable"
                         msg += ", which reduces performance by ~70%."
-                        msg += "Its recommended to put git.exe into the PATH or to "
+                        msg += "It is recommended to put git.exe into the PATH or to "
                         msg += "set the %s " % self._git_exec_env_var
                         msg += "environment variable to the executable's location"
                         warnings.warn(msg)
@@ -769,10 +875,10 @@ class Git(LazyMixin):
         return (hexsha, typename, size, data)
 
     def stream_object_data(self, ref):
-        """As get_object_header, but returns the data as a stream
+        """ As get_object_header, but returns the data as a stream
+
         :return: (hexsha, type_string, size_as_int, stream)
-        :note: This method is not threadsafe, you need one independent  Command instance
-            per thread to be safe !"""
+        :note: This method is not threadsafe, you need one independent Command instance per thread to be safe !"""
         cmd = self._get_persistent_cmd("cat_file_all", "cat_file", batch=True)
         hexsha, typename, size = self.__get_object_header(cmd, ref)
         return (hexsha, typename, size, self.CatFileContentStream(size, cmd.stdout))
