@@ -14,6 +14,7 @@ from .config import (
     cp,
 )
 from .refs import (
+    Head,
     Reference,
     RemoteReference,
     SymbolicReference,
@@ -177,8 +178,9 @@ class FetchInfo(object):
      info.note          # additional notes given by git-fetch intended for the user
      info.old_commit    # if info.flags & info.FORCED_UPDATE|info.FAST_FORWARD,
                         # field is set to the previous location of ref, otherwise None
+     info.remote_ref_path # The path from which we fetched on the remote. It's the remote's version of our info.ref
     """
-    __slots__ = ('ref', 'old_commit', 'flags', 'note')
+    __slots__ = ('ref', 'old_commit', 'flags', 'note', 'remote_ref_path')
 
     NEW_TAG, NEW_HEAD, HEAD_UPTODATE, TAG_UPDATE, REJECTED, FORCED_UPDATE, \
         FAST_FORWARD, ERROR = [1 << x for x in range(8)]
@@ -193,7 +195,7 @@ class FetchInfo(object):
                  '=': HEAD_UPTODATE,
                  ' ': FAST_FORWARD}
 
-    def __init__(self, ref, flags, note='', old_commit=None):
+    def __init__(self, ref, flags, note='', old_commit=None, remote_ref_path=None):
         """
         Initialize a new instance
         """
@@ -201,6 +203,7 @@ class FetchInfo(object):
         self.flags = flags
         self.note = note
         self.old_commit = old_commit
+        self.remote_ref_path = remote_ref_path
 
     def __str__(self):
         return self.name
@@ -243,7 +246,7 @@ class FetchInfo(object):
             new_hex_sha, fetch_operation, fetch_note = fetch_line.split("\t")
             ref_type_name, fetch_note = fetch_note.split(' ', 1)
         except ValueError:  # unpack error
-            raise ValueError("Failed to parse FETCH__HEAD line: %r" % fetch_line)
+            raise ValueError("Failed to parse FETCH_HEAD line: %r" % fetch_line)
 
         # parse flags from control_character
         flags = 0
@@ -288,6 +291,11 @@ class FetchInfo(object):
             # note: remote-tracking is just the first part of the 'remote-tracking branch' token.
             # We don't parse it correctly, but its enough to know what to do, and its new in git 1.7something
             ref_type = RemoteReference
+        elif '/' in ref_type_name:
+            # If the fetch spec look something like this '+refs/pull/*:refs/heads/pull/*', and is thus pretty
+            # much anything the user wants, we will have trouble to determine what's going on
+            # For now, we assume the local ref is a Head
+            ref_type = Head
         else:
             raise TypeError("Cannot handle reference type: %r" % ref_type_name)
         # END handle ref type
@@ -325,7 +333,7 @@ class FetchInfo(object):
 
         note = (note and note.strip()) or ''
 
-        return cls(remote_local_ref, flags, note, old_commit)
+        return cls(remote_local_ref, flags, note, old_commit, local_remote_ref)
 
 
 class Remote(LazyMixin, Iterable):
@@ -354,14 +362,14 @@ class Remote(LazyMixin, Iterable):
             # that it has the config_writer property, but instead calls __getattr__
             # which will not yield the expected results. 'pinging' the members
             # with a dir call creates the config_writer property that we require
-            # ... bugs like these make me wonder wheter python really wants to be used
+            # ... bugs like these make me wonder whether python really wants to be used
             # for production. It doesn't happen on linux though.
             dir(self)
         # END windows special handling
 
     def __getattr__(self, attr):
         """Allows to call this instance like
-        remote.special( *args, **kwargs) to call git-remote special self.name"""
+        remote.special( \*args, \*\*kwargs) to call git-remote special self.name"""
         if attr == "_config_reader":
             return super(Remote, self).__getattr__(attr)
 
@@ -378,6 +386,8 @@ class Remote(LazyMixin, Iterable):
 
     def _set_cache_(self, attr):
         if attr == "_config_reader":
+            # NOTE: This is cached as __getattr__ is overridden to return remote config values implicitly, such as
+            # in print(r.pushurl)
             self._config_reader = SectionConstraint(self.repo.config_reader(), self._config_section_name())
         else:
             super(Remote, self)._set_cache_(attr)
@@ -398,8 +408,9 @@ class Remote(LazyMixin, Iterable):
         return hash(self.name)
 
     def exists(self):
-        """:return: True if this is a valid, existing remote.
-        Valid remotes have an entry in the repository's configuration"""
+        """
+        :return: True if this is a valid, existing remote.
+            Valid remotes have an entry in the repository's configuration"""
         try:
             self.config_reader.get('url')
             return True
@@ -429,7 +440,7 @@ class Remote(LazyMixin, Iterable):
         :return:
             IterableList of RemoteReference objects. It is prefixed, allowing
             you to omit the remote path portion, i.e.::
-             remote.refs.master # yields RemoteReference('/refs/remotes/origin/master')"""
+            remote.refs.master # yields RemoteReference('/refs/remotes/origin/master')"""
         out_refs = IterableList(RemoteReference._id_attribute_, "%s/" % self.name)
         out_refs.extend(RemoteReference.list_items(self.repo, remote=self.name))
         assert out_refs, "Remote %s did not have any references" % self.name
@@ -463,11 +474,8 @@ class Remote(LazyMixin, Iterable):
         :param repo: Repository instance that is to receive the new remote
         :param name: Desired name of the remote
         :param url: URL which corresponds to the remote's name
-        :param kwargs:
-            Additional arguments to be passed to the git-remote add command
-
+        :param kwargs: Additional arguments to be passed to the git-remote add command
         :return: New Remote instance
-
         :raise GitCommandError: in case an origin with that name already exists"""
         repo.git.remote("add", name, url, **kwargs)
         return cls(repo, name)
@@ -477,8 +485,13 @@ class Remote(LazyMixin, Iterable):
 
     @classmethod
     def remove(cls, repo, name):
-        """Remove the remote with the given name"""
+        """Remove the remote with the given name
+        :return: the passed remote name to remove
+        """
         repo.git.remote("rm", name)
+        if isinstance(name, cls):
+            name._clear_cache()
+        return name
 
     # alias
     rm = remove
@@ -491,11 +504,8 @@ class Remote(LazyMixin, Iterable):
 
         self.repo.git.remote("rename", self.name, new_name)
         self.name = new_name
-        try:
-            del(self._config_reader)        # it contains cached values, section names are different now
-        except AttributeError:
-            pass
-        # END handle exception
+        self._clear_cache()
+
         return self
 
     def update(self, **kwargs):
@@ -525,7 +535,9 @@ class Remote(LazyMixin, Iterable):
 
         progress_handler = progress.new_message_handler()
 
-        def my_progress_handler(line):
+        for line in proc.stderr:
+            line = line.decode(defenc)
+            line = line.rstrip()
             for pline in progress_handler(line):
                 if line.startswith('fatal:'):
                     raise GitCommandError(("Error when fetching: %s" % line,), 2)
@@ -540,7 +552,7 @@ class Remote(LazyMixin, Iterable):
         # end
 
         # We are only interested in stderr here ...
-        handle_process_output(proc, None, my_progress_handler, finalize_process)
+        finalize_process(proc)
 
         # read head information
         fp = open(join(self.repo.git_dir, 'FETCH_HEAD'), 'rb')
@@ -610,6 +622,7 @@ class Remote(LazyMixin, Iterable):
             args = [refspec]
 
         proc = self.repo.git.fetch(self, *args, with_extended_output=True, as_process=True, v=True, **kwargs)
+        proc.stdout.close()
         res = self._get_fetch_info_from_stderr(proc, progress or RemoteProgress())
         if hasattr(self.repo.odb, 'update_cache'):
             self.repo.odb.update_cache()
@@ -625,6 +638,7 @@ class Remote(LazyMixin, Iterable):
         :return: Please see 'fetch' method """
         kwargs = add_progress(kwargs, self.repo.git, progress)
         proc = self.repo.git.pull(self, refspec, with_extended_output=True, as_process=True, v=True, **kwargs)
+        proc.stdout.close()
         res = self._get_fetch_info_from_stderr(proc, progress or RemoteProgress())
         if hasattr(self.repo.odb, 'update_cache'):
             self.repo.odb.update_cache()
@@ -660,6 +674,13 @@ class Remote(LazyMixin, Iterable):
             Hence you may simple type config.get("pushurl") to obtain the information"""
         return self._config_reader
 
+    def _clear_cache(self):
+        try:
+            del(self._config_reader)
+        except AttributeError:
+            pass
+        # END handle exception
+
     @property
     def config_writer(self):
         """
@@ -674,9 +695,5 @@ class Remote(LazyMixin, Iterable):
         writer = self.repo.config_writer()
 
         # clear our cache to assure we re-read the possibly changed configuration
-        try:
-            del(self._config_reader)
-        except AttributeError:
-            pass
-        # END handle exception
+        self._clear_cache()
         return SectionConstraint(writer, self._config_section_name())
