@@ -4,7 +4,11 @@
 # This module is part of GitPython and is released under
 # the BSD License: http://www.opensource.org/licenses/bsd-license.php
 
-from git.exc import InvalidGitRepositoryError, NoSuchPathError
+from git.exc import (
+    InvalidGitRepositoryError,
+    NoSuchPathError,
+    GitCommandError
+)
 from git.cmd import (
     Git,
     handle_process_output
@@ -71,7 +75,6 @@ def _expand_path(p):
 
 
 class Repo(object):
-
     """Represents a git repository and allows you to query references,
     gather commit information, generate diffs, create and clone repositories query
     the log.
@@ -99,17 +102,23 @@ class Repo(object):
     # represents the configuration level of a configuration file
     config_level = ("system", "user", "global", "repository")
 
+    # Subclass configuration
+    # Subclasses may easily bring in their own custom types by placing a constructor or type here
+    GitCommandWrapperType = Git
+
     def __init__(self, path=None, odbt=DefaultDBType, search_parent_directories=False):
         """Create a new Repo instance
 
-        :param path: is the path to either the root git directory or the bare git repo::
+        :param path:
+            the path to either the root git directory or the bare git repo::
 
-            repo = Repo("/Users/mtrier/Development/git-python")
-            repo = Repo("/Users/mtrier/Development/git-python.git")
-            repo = Repo("~/Development/git-python.git")
-            repo = Repo("$REPOSITORIES/Development/git-python.git")
+                repo = Repo("/Users/mtrier/Development/git-python")
+                repo = Repo("/Users/mtrier/Development/git-python.git")
+                repo = Repo("~/Development/git-python.git")
+                repo = Repo("$REPOSITORIES/Development/git-python.git")
 
-        :param odbt: Object DataBase type - a type which is constructed by providing
+        :param odbt:
+            Object DataBase type - a type which is constructed by providing
             the directory containing the database objects, i.e. .git/objects. It will
             be used to access all object data
         :raise InvalidGitRepositoryError:
@@ -127,14 +136,18 @@ class Repo(object):
 
         # walk up the path to find the .git dir
         while curpath:
+            # ABOUT os.path.NORMPATH
+            # It's important to normalize the paths, as submodules will otherwise initialize their
+            # repo instances with paths that depend on path-portions that will not exist after being
+            # removed. It's just cleaner.
             if is_git_dir(curpath):
-                self.git_dir = curpath
+                self.git_dir = os.path.normpath(curpath)
                 self._working_tree_dir = os.path.dirname(self.git_dir)
                 break
 
             gitpath = find_git_dir(join(curpath, '.git'))
             if gitpath is not None:
-                self.git_dir = gitpath
+                self.git_dir = os.path.normpath(gitpath)
                 self._working_tree_dir = curpath
                 break
 
@@ -162,7 +175,7 @@ class Repo(object):
         # END working dir handling
 
         self.working_dir = self._working_tree_dir or self.git_dir
-        self.git = Git(self.working_dir)
+        self.git = self.GitCommandWrapperType(self.working_dir)
 
         # special handling, in special times
         args = [join(self.git_dir, 'objects')]
@@ -201,10 +214,8 @@ class Repo(object):
 
     @property
     def working_tree_dir(self):
-        """:return: The working tree directory of our git repository
-        :raise AssertionError: If we are a bare repository"""
-        if self._working_tree_dir is None:
-            raise AssertionError("Repository at %r is bare and does not have a working tree directory" % self.git_dir)
+        """:return: The working tree directory of our git repository. If this is a bare repository, None is returned.
+        """
         return self._working_tree_dir
 
     @property
@@ -355,11 +366,11 @@ class Repo(object):
             return "/etc/gitconfig"
         elif config_level == "user":
             config_home = os.environ.get("XDG_CONFIG_HOME") or os.path.join(os.environ.get("HOME", '~'), ".config")
-            return os.path.expanduser(join(config_home, "git", "config"))
+            return os.path.normpath(os.path.expanduser(join(config_home, "git", "config")))
         elif config_level == "global":
             return os.path.normpath(os.path.expanduser("~/.gitconfig"))
         elif config_level == "repository":
-            return join(self.git_dir, "config")
+            return os.path.normpath(join(self.git_dir, "config"))
 
         raise ValueError("Invalid configuration level: %r" % config_level)
 
@@ -456,6 +467,37 @@ class Repo(object):
 
         return Commit.iter_items(self, rev, paths, **kwargs)
 
+    def merge_base(self, *rev, **kwargs):
+        """Find the closest common ancestor for the given revision (e.g. Commits, Tags, References, etc)
+
+        :param rev: At least two revs to find the common ancestor for.
+        :param kwargs: Additional arguments to be passed to the repo.git.merge_base() command which does all the work.
+        :return: A list of Commit objects. If --all was not specified as kwarg, the list will have at max one Commit,
+            or is empty if no common merge base exists.
+        :raises ValueError: If not at least two revs are provided
+        """
+        if len(rev) < 2:
+            raise ValueError("Please specify at least two revs, got only %i" % len(rev))
+        # end handle input
+
+        res = list()
+        try:
+            lines = self.git.merge_base(*rev, **kwargs).splitlines()
+        except GitCommandError as err:
+            if err.status == 128:
+                raise
+            # end handle invalid rev
+            # Status code 1 is returned if there is no merge-base
+            # (see https://github.com/git/git/blob/master/builtin/merge-base.c#L16)
+            return res
+        # end exception handling
+
+        for line in lines:
+            res.append(self.commit(line))
+        # end for each merge-base
+
+        return res
+
     def _get_daemon_export(self):
         filename = join(self.git_dir, self.DAEMON_EXPORT_FILE)
         return os.path.exists(filename)
@@ -715,7 +757,7 @@ class Repo(object):
         # git command automatically chdir into the directory
         git = Git(path)
         git.init(**kwargs)
-        return Repo(path)
+        return cls(path)
 
     @classmethod
     def _clone(cls, git, url, path, odb_default_type, progress, **kwargs):
@@ -780,16 +822,13 @@ class Repo(object):
 
     def clone(self, path, progress=None, **kwargs):
         """Create a clone from this repository.
-        :param path:
-            is the full path of the new repo (traditionally ends with ./<name>.git).
 
+        :param path: is the full path of the new repo (traditionally ends with ./<name>.git).
         :param progress: See 'git.remote.Remote.push'.
-
         :param kwargs:
-            odbt = ObjectDatabase Type, allowing to determine the object database
-            implementation used by the returned Repo instance
-
-            All remaining keyword arguments are given to the git-clone command
+            * odbt = ObjectDatabase Type, allowing to determine the object database
+              implementation used by the returned Repo instance
+            * All remaining keyword arguments are given to the git-clone command
 
         :return: ``git.Repo`` (the newly cloned repo)"""
         return self._clone(self.git, self.git_dir, path, type(self.odb), progress, **kwargs)
@@ -806,13 +845,16 @@ class Repo(object):
 
     def archive(self, ostream, treeish=None, prefix=None, **kwargs):
         """Archive the tree at the given revision.
+
         :parm ostream: file compatible stream object to which the archive will be written as bytes
         :parm treeish: is the treeish name/id, defaults to active branch
         :parm prefix: is the optional prefix to prepend to each filename in the archive
-        :parm kwargs:
-            Additional arguments passed to git-archive
-            NOTE: Use the 'format' argument to define the kind of format. Use
-            specialized ostreams to write any format supported by python
+        :parm kwargs: Additional arguments passed to git-archive
+
+            * Use the 'format' argument to define the kind of format. Use
+              specialized ostreams to write any format supported by python.
+            * You may specify the special **path** keyword, which may either be a repository-relative
+              path to a directory or file to place into the archive, or a list or tuple of multipe paths.
 
         :raise GitCommandError: in case something went wrong
         :return: self"""
@@ -821,10 +863,24 @@ class Repo(object):
         if prefix and 'prefix' not in kwargs:
             kwargs['prefix'] = prefix
         kwargs['output_stream'] = ostream
+        path = kwargs.pop('path', list())
+        if not isinstance(path, (tuple, list)):
+            path = [path]
+        # end assure paths is list
 
-        self.git.archive(treeish, **kwargs)
+        self.git.archive(treeish, *path, **kwargs)
         return self
 
+    def has_separate_working_tree(self):
+        """
+        :return: True if our git_dir is not at the root of our working_tree_dir, but a .git file with a
+            platform agnositic symbolic link. Our git_dir will be whereever the .git file points to
+        :note: bare repositories will always return False here
+        """
+        if self.bare:
+            return False
+        return os.path.isfile(os.path.join(self.working_tree_dir, '.git'))
+
     rev_parse = rev_parse
 
     def __repr__(self):
