@@ -26,6 +26,7 @@ __all__ = ('rev_parse', 'is_git_dir', 'touch', 'find_git_dir', 'name_to_object',
 def touch(filename):
     fp = open(filename, "ab")
     fp.close()
+    return filename
 
 
 def is_git_dir(d):
@@ -147,8 +148,6 @@ def rev_parse(repo, rev):
     :param rev: git-rev-parse compatible revision specification as string, please see
         http://www.kernel.org/pub/software/scm/git/docs/git-rev-parse.html
         for details
-    :note: Currently there is no access to the rev-log, rev-specs may only contain
-        topological tokens such ~ and ^.
     :raise BadObject: if the given revision could not be found
     :raise ValueError: If rev couldn't be parsed
     :raise IndexError: If invalid reflog index is specified"""
@@ -296,7 +295,7 @@ def rev_parse(repo, rev):
                 raise ValueError("Invalid token: %r" % token)
             # END end handle tag
         except (IndexError, AttributeError):
-            raise BadObject("Invalid Revision in %s" % rev)
+            raise BadName("Invalid revision spec '%s' - not enough parent commits to reach '%s%i'" % (rev, token, num))
         # END exception handling
     # END parse loop
 
