@@ -1,12 +1,11 @@
 """Performance tests for object store"""
-
+from __future__ import print_function
 from time import time
 import sys
-import stat
 
-from lib import (
+from .lib import (
     TestBigRepoR
-    )
+)
 
 
 class TestObjDBPerformance(TestBigRepoR):
@@ -16,12 +15,13 @@ class TestObjDBPerformance(TestBigRepoR):
         for repo in (self.gitrorepo, self.puregitrorepo):
             # GET COMMITS
             st = time()
-            root_commit = repo.commit(self.head_sha_2k)
+            root_commit = repo.commit(repo.head)
             commits = list(root_commit.traverse())
             nc = len(commits)
             elapsed = time() - st
 
-            print >> sys.stderr, "%s: Retrieved %i commits from ObjectStore in %g s ( %f commits / s )" % (type(repo.odb), nc, elapsed, nc / elapsed)
+            print("%s: Retrieved %i commits from ObjectStore in %g s ( %f commits / s )"
+                  % (type(repo.odb), nc, elapsed, nc / elapsed), file=sys.stderr)
             results[0].append(elapsed)
 
             # GET TREES
@@ -42,7 +42,8 @@ class TestObjDBPerformance(TestBigRepoR):
             # END for each commit
             elapsed = time() - st
 
-            print >> sys.stderr, "%s: Retrieved %i objects from %i commits in %g s ( %f objects / s )" % (type(repo.odb), nt, len(commits), elapsed, nt / elapsed)
+            print("%s: Retrieved %i objects from %i commits in %g s ( %f objects / s )"
+                  % (type(repo.odb), nt, len(commits), elapsed, nt / elapsed), file=sys.stderr)
             results[1].append(elapsed)
 
             # GET BLOBS
@@ -60,11 +61,13 @@ class TestObjDBPerformance(TestBigRepoR):
             # END for each bloblist
             elapsed = time() - st
 
-            print >> sys.stderr, "%s: Retrieved %i blob (%i KiB) and their data in %g s ( %f blobs / s, %f KiB / s )" % (type(repo.odb), nb, data_bytes / 1000, elapsed, nb / elapsed, (data_bytes / 1000) / elapsed)
+            msg = "%s: Retrieved %i blob (%i KiB) and their data in %g s ( %f blobs / s, %f KiB / s )"\
+                % (type(repo.odb), nb, data_bytes / 1000, elapsed, nb / elapsed, (data_bytes / 1000) / elapsed)
+            print(msg, file=sys.stderr)
             results[2].append(elapsed)
         # END for each repo type
 
         # final results
         for test_name, a, b in results:
-            print >> sys.stderr, "%s: %f s vs %f s, pure is %f times slower" % (test_name, a, b, b / a)
+            print("%s: %f s vs %f s, pure is %f times slower" % (test_name, a, b, b / a), file=sys.stderr)
         # END for each result
