@@ -7,7 +7,8 @@
 from git.test.lib import (
     TestBase,
     with_rw_repo,
-    with_rw_and_rw_remote_repo
+    with_rw_and_rw_remote_repo,
+    fixture
 )
 from git import (
     RemoteProgress,
@@ -163,11 +164,11 @@ class TestRemote(TestBase):
         def get_info(res, remote, name):
             return res["%s/%s" % (remote, name)]
 
-        # put remote head to master as it is garantueed to exist
+        # put remote head to master as it is guaranteed to exist
         remote_repo.head.reference = remote_repo.heads.master
 
         res = fetch_and_test(remote)
-        # all uptodate
+        # all up to date
         for info in res:
             assert info.flags & info.HEAD_UPTODATE
 
@@ -348,6 +349,7 @@ class TestRemote(TestBase):
         new_head = Head.create(rw_repo, "my_new_branch")
         progress = TestRemoteProgress()
         res = remote.push(new_head, progress)
+        assert len(res) > 0
         assert res[0].flags & PushInfo.NEW_HEAD
         progress.make_assertion()
         self._do_test_push_result(res, remote)
@@ -450,11 +452,15 @@ class TestRemote(TestBase):
         remote = Remote.create(bare_rw_repo, *arg_list)
         assert remote.name == "test_new_one"
         assert remote in bare_rw_repo.remotes
+        assert remote.exists()
 
         # create same one again
         self.failUnlessRaises(GitCommandError, Remote.create, bare_rw_repo, *arg_list)
 
         Remote.remove(bare_rw_repo, new_name)
+        assert remote.exists()      # We still have a cache that doesn't know we were deleted by name
+        remote._clear_cache()
+        assert not remote.exists()  # Cache should be renewed now. This is an issue ...
 
         for remote in bare_rw_repo.remotes:
             if remote.name == new_name:
@@ -515,3 +521,16 @@ class TestRemote(TestBase):
 
         assert type(fi.ref) is Reference
         assert fi.ref.path == "refs/something/branch"
+
+    def test_uncommon_branch_names(self):
+        stderr_lines = fixture('uncommon_branch_prefix_stderr').decode('ascii').splitlines()
+        fetch_lines = fixture('uncommon_branch_prefix_FETCH_HEAD').decode('ascii').splitlines()
+
+        # The contents of the files above must be fetched with a custom refspec:
+        # +refs/pull/*:refs/heads/pull/*
+        res = [FetchInfo._from_line('ShouldntMatterRepo', stderr, fetch_line)
+               for stderr, fetch_line in zip(stderr_lines, fetch_lines)]
+        assert len(res)
+        assert res[0].remote_ref_path == 'refs/pull/1/head'
+        assert res[0].ref.path == 'refs/heads/pull/1/head'
+        assert isinstance(res[0].ref, Head)
